#!/usr/local/bin/perl

#Web Gallery Ver 2.06 (2007/09/14)
#
#Copyright(C) 2002-2007 Knight, All rights reserved.
#Mail ... support@web-liberty.net
#Home ... http://www.web-liberty.net/

#――――― 初期処理 ――――――――――――――――――――――――

package main;

use strict;
use lib qw(./lib);

use webliberty::Basis;
use webliberty::Parser;
use webliberty::String;
use webliberty::Encoder;
use webliberty::File;
use webliberty::Date;
use webliberty::Cookie;
use webliberty::Lock;
use webliberty::Skin;

require './init.cgi';

#――――― メイン処理 ―――――――――――――――――――――――

my $init = &init::get_init;

my $basis_ins  = new webliberty::Basis;
my $parser_ins = new webliberty::Parser(max => $init->{parse_size}, jcode => $init->{jcode_mode});

my $agent;
if ($ENV{'HTTP_USER_AGENT'} =~ /(DoCoMo|SoftBank|Vodafone|J-PHONE|KDDI-|UP\.Browser|DDIPOCKET|WILLCOM)/i) {
	require Jcode;

	$agent = 'mobile';
}

if ($parser_ins->get_query('mode') eq 'admin') {
	if (&check_password) {
		if ($parser_ins->get_query('work') eq 'logout') {
			&logout;
		} elsif ($parser_ins->get_query('work') eq 'new') {
			if ($parser_ins->get_query('exec_regist')) {
				&regist;
				&edit;
			} else {
				&form;
			}
		} elsif ($parser_ins->get_query('work') eq 'edit') {
			if ($parser_ins->get_query('exec_regist')) {
				&regist;
			} elsif ($parser_ins->get_query('exec_del')) {
				&del;
			} elsif ($parser_ins->get_query('exec_order')) {
				&order;
			}
			if ($parser_ins->get_query('exec_form')) {
				&form;
			} elsif ($parser_ins->get_query('exec_confirm')) {
				&confirm;
			} else {
				&edit;
			}
		} elsif ($parser_ins->get_query('work') eq 'field') {
			if ($parser_ins->get_query('exec_add')) {
				&add_field;
			} elsif ($parser_ins->get_query('exec_edit')) {
				&edit_field;
			}
			&field;
		} elsif ($parser_ins->get_query('work') eq 'file') {
			if ($parser_ins->get_query('exec_add')) {
				&add_file;
			} elsif ($parser_ins->get_query('exec_edit')) {
				&edit_file;
			}
			&file;
		} elsif ($parser_ins->get_query('work') eq 'top') {
			if ($parser_ins->get_query('exec_regist')) {
				&update;
			}
			&top;
		} else {
			&edit;
		}
	} else {
		&login;
	}
} elsif ($parser_ins->get_query('mode') eq 'search') {
	&search;
} elsif ($parser_ins->get_query('mode') eq 'view') {
	&view;
} elsif ($parser_ins->get_query('mode') eq 'list') {
	&list;
} else {
	&gallery;
}

exit;

#――――― サブルーチン ――――――――――――――――――――――

### パスワード認証
sub check_password {
	my $cookie_ins = new webliberty::Cookie($init->{cookie_admin}, $init->{des_key});

	my(%query, $admin_pwd);

	if ($parser_ins->get_query) {
		%query = %{$parser_ins->get_query};
	}

	if (exists $query{'admin_pwd'}) {
		$admin_pwd = $parser_ins->get_query('admin_pwd');
	} else {
		$admin_pwd = $cookie_ins->get_cookie('admin_pwd');
	}

	my $flag;

	if ($admin_pwd eq $init->{admin_pwd}) {
		if ($parser_ins->get_query('admin_pwd')) {
			if ($parser_ins->get_query('hold')) {
				$cookie_ins->set_holddays(3650);
			}
			$cookie_ins->set_cookie(
				admin_pwd => $admin_pwd
			);
		}

		$flag = 1;
	}

	return $flag;
}

### ログアウト
sub logout {
	my $cookie_ins = new webliberty::Cookie($init->{cookie_admin}, $init->{des_key});
	$cookie_ins->set_cookie(
		admin_pwd => ''
	);

	&login;

	return;
}

### 作品登録
sub regist {
	if ($ENV{'REQUEST_METHOD'} ne 'POST') {
		&error('不正なアクセスです。');
	}

	my $edit_ins  = new webliberty::String($parser_ins->get_query('edit'));
	my $id_ins    = new webliberty::String($parser_ins->get_query('id'));
	my $stat_ins  = new webliberty::String($parser_ins->get_query('stat'));
	my $break_ins = new webliberty::String($parser_ins->get_query('break'));
	my $field_ins = new webliberty::String($parser_ins->get_query('field'));
	my $tag_ins   = new webliberty::String($parser_ins->get_query('tag'));
	my $subj_ins  = new webliberty::String($parser_ins->get_query('subj'));
	my $memo_ins  = new webliberty::String($parser_ins->get_query('memo'));
	my $text_ins  = new webliberty::String($parser_ins->get_query('text'));

	$edit_ins->create_line;
	$id_ins->create_line;
	$stat_ins->create_number;
	$break_ins->create_number;
	$field_ins->create_line;
	$tag_ins->create_line;
	$subj_ins->create_line;
	$memo_ins->create_text;
	$text_ins->create_text;

	$tag_ins->replace_string('\s*,\s*', '<>');

	if (!$subj_ins->get_string) {
		&error('題名が入力されていません。');
	}
	if (!$text_ins->get_string) {
		&error('本文が入力されていません。');
	}

	if ($id_ins->get_string =~ /[^\w\d\-\_]/) {
		&error("IDは半角英数字で指定してください。");
	}
	if (!($parser_ins->get_query('year') =~ /^\d\d\d\d$/)) {
		&error("日付の年が正しく指定されていません。");
	}
	if (!($parser_ins->get_query('month') =~ /^\d\d$/)) {
		&error("日付の月が正しく指定されていません。");
	}
	if (!($parser_ins->get_query('day') =~ /^\d\d$/)) {
		&error("日付の日が正しく指定されていません。");
	}
	if (!($parser_ins->get_query('hour') =~ /^\d\d$/)) {
		&error("日付の時間が正しく指定されていません。");
	}
	if (!($parser_ins->get_query('minute') =~ /^\d\d$/)) {
		&error("日付の分が正しく指定されていません。");
	}

	my $lock_ins = new webliberty::Lock($init->{lock_file});
	if (!$lock_ins->file_lock) {
		&error('ファイルがロックされています。時間をおいてもう一度登録してください。');
	}

	if ($edit_ins->get_string) {
		#作品編集
		my($new_data, $flag);

		open(FH, $init->{log_file}) or &error("Read Error : $init->{log_file}");
		while (<FH>) {
			chomp;
			my($id, $stat, $break, $field, $tag, $date, $subj, $memo, $text) = split(/\t/);

			if ($edit_ins->get_string eq $id) {
				if ($id_ins->get_string) {
					$id = $id_ins->get_string;
				}

				$new_data .= "$id\t" . $stat_ins->get_string . "\t" . $break_ins->get_string . "\t" . $field_ins->get_string . "\t" . $tag_ins->get_string . "\t" . $parser_ins->get_query('year') . $parser_ins->get_query('month') . $parser_ins->get_query('day') . $parser_ins->get_query('hour') . $parser_ins->get_query('minute') . "\t" . $subj_ins->get_string . "\t" . $memo_ins->get_string . "\t" . $text_ins->get_string . "\n";

				$flag = 1;
			} else {
				if ($id_ins->get_string and $id_ins->get_string eq $id) {
					$lock_ins->file_unlock;
					&error("そのIDは既に使用されています。");
				}

				$new_data .= "$_\n";
			}
		}
		close(FH);

		if (!$flag) {
			$lock_ins->file_unlock;
			&error('該当する作品はありません。');
		}

		open(FH, ">$init->{log_file}") or &error("Write Error : $init->{log_file}");
		print FH $new_data;
		close(FH);
	} else {
		#ID重複チェック
		my $new_id;
		if ($id_ins->get_string) {
			$new_id = $id_ins->get_string;
		} else {
			$new_id = time;
		}
		open(FH, $init->{log_file}) or &error("Read Error : $init->{log_file}");
		while (<FH>) {
			chomp;
			my($id, $stat, $break, $field, $tag, $date, $subj, $memo, $text) = split(/\t/);

			if ($new_id eq $id) {
				$lock_ins->file_unlock;
				if ($init->{use_id}) {
					&error("そのIDは既に使用されています。");
				} else {
					&error("IDの発行に失敗しました。");
				}
			}
		}
		close(FH);

		#作品登録
		open(FH, $init->{log_file}) or &error("Read Error : $init->{log_file}");
		my @logs = <FH>;
		close(FH);

		push(@logs, "$new_id\t" . $stat_ins->get_string . "\t" . $break_ins->get_string . "\t" . $field_ins->get_string . "\t" . $tag_ins->get_string . "\t" . $parser_ins->get_query('year') . $parser_ins->get_query('month') . $parser_ins->get_query('day') . $parser_ins->get_query('hour') . $parser_ins->get_query('minute') . "\t" . $subj_ins->get_string . "\t" . $memo_ins->get_string . "\t" . $text_ins->get_string . "\n");

		@logs = &sort_item(@logs);

		open(FH, ">$init->{log_file}") or &error("Write Error : $init->{log_file}");
		print FH @logs;
		close(FH);
	}

	#タグ情報更新
	&update_tags;

	$lock_ins->file_unlock;

	return;
}

### 作品削除
sub del {
	if ($ENV{'REQUEST_METHOD'} ne 'POST') {
		&error('不正なアクセスです。');
	}
	if (!$parser_ins->get_query('del')) {
		&error('削除したい作品を選択してください。');
	}

	my $lock_ins = new webliberty::Lock($init->{lock_file});
	if (!$lock_ins->file_lock) {
		&error('ファイルがロックされています。時間をおいてもう一度登録してください。');
	}

	my($new_data, $flag);

	open(FH, $init->{log_file}) or &error("Read Error : $init->{log_file}");
	while (<FH>) {
		chomp;
		my($id, $stat, $break, $field, $tag, $date, $subj, $memo, $text) = split(/\t/);

		if ($parser_ins->get_query('del') =~ /(^|\n)$id(\n|$)/) {
			$flag = 1;
		} else {
			$new_data .= "$_\n";
		}
	}
	close(FH);

	if (!$flag) {
		$lock_ins->file_unlock;
		&error('指定された作品は存在しません。');
	}

	open(FH, ">$init->{log_file}") or &error("Write Error : $init->{log_file}");
	print FH $new_data;
	close(FH);

	#タグ情報更新
	&update_tags;

	$lock_ins->file_unlock;

	return;
}

### タグ情報更新
sub update_tags {
	my %tags;
	open(FH, $init->{log_file}) or &error("Read Error : $init->{log_file}");
	while (<FH>) {
		chomp;
		my($id, $stat, $break, $field, $tag, $date, $subj, $memo, $text) = split(/\t/);

		if ($tag) {
			foreach my $word (split(/<>/, $tag)) {
				$tags{$word}++;
			}
		}
	}
	close(FH);

	my $new_data;
	foreach (sort { $a cmp $b } keys %tags) {
		$new_data .= "$_\t$tags{$_}\n";
	}

	open(FH, ">$init->{tag_file}") or &error("Write Error : $init->{tag_file}");
	print FH $new_data;
	close(FH);

	return;
}

### 作品並び替え
sub order {
	if ($ENV{'REQUEST_METHOD'} ne 'POST') {
		&error('不正なアクセスです。');
	}
	if (!$parser_ins->get_query('from') or !$parser_ins->get_query('to')) {
		&error('移動元と移動先を選択してください。');
	}

	my $lock_ins = new webliberty::Lock($init->{lock_file});
	if (!$lock_ins->file_lock) {
		&error('ファイルがロックされています。時間をおいてもう一度登録してください。');
	}

	my(@new_data, $to_data, $i);

	open(FH, $init->{log_file}) or &error("Read Error : $init->{log_file}");
	while (<FH>) {
		chomp;
		my($id, $stat, $break, $field, $tag, $date, $subj, $memo, $text) = split(/\t/);

		if ($parser_ins->get_query('from') eq $id) {
			$to_data = $_;
		}
	}
	seek(FH, 0, 0);
	while (<FH>) {
		chomp;
		my($id, $stat, $break, $field, $tag, $date, $subj, $memo, $text) = split(/\t/);

		if ($parser_ins->get_query('from') ne $id) {
			push(@new_data, "$_\n");
		}
		if ($parser_ins->get_query('to') eq $id) {
			push(@new_data, "$to_data\n");
		}
	}
	close(FH);

	@new_data = &sort_item(@new_data);

	open(FH, ">$init->{log_file}") or &error("Write Error : $init->{log_file}");
	print FH @new_data;
	close(FH);

	$lock_ins->file_unlock;

	return;
}

### 分類追加
sub add_field {
	if ($ENV{'REQUEST_METHOD'} ne 'POST') {
		&error('不正なアクセスです。');
	}

	my $id_ins    = new webliberty::String($parser_ins->get_query('id'));
	my $field_ins = new webliberty::String($parser_ins->get_query('field'));

	$id_ins->create_line;
	$field_ins->create_line;

	if (!$id_ins->get_string) {
		&error('分類IDを入力してください。');
	}
	if (!$field_ins->get_string) {
		&error('分類名を入力してください。');
	}

	if ($id_ins->get_string =~ /[^\w\d\-\_]/) {
		&error('分類IDは半角英数字で指定してください。');
	}
	if ($id_ins->get_string eq '_') {
		&error('分類ID「_」を使用することはできません。');
	}

	my $lock_ins = new webliberty::Lock($init->{lock_file});
	if (!$lock_ins->file_lock) {
		&error('ファイルがロックされています。時間をおいてもう一度登録してください。');
	}

	open(FH, ">>$init->{field_file}") or &error("Write Error : $init->{field_file}");
	print FH $id_ins->get_string . "\t" . $field_ins->get_string . "\n";
	close(FH);

	$lock_ins->file_unlock;

	return;
}

### 分類編集
sub edit_field {
	if ($ENV{'REQUEST_METHOD'} ne 'POST') {
		&error('不正なアクセスです。');
	}

	my $lock_ins = new webliberty::Lock($init->{lock_file});
	if (!$lock_ins->file_lock) {
		&error('ファイルがロックされています。時間をおいてもう一度登録してください。');
	}

	my(@fields, $new_data);
	my $i = 0;

	open(FH, $init->{field_file}) or &error("Read Error : $init->{field_file}");
	while (<FH>) {
		push(@fields, $_);
	}
	seek(FH, 0, 0);
	while (<FH>) {
		chomp;
		my($id, $field) = split(/\t/);

		if ($parser_ins->get_query('from') ne '' and $parser_ins->get_query('to') ne '') {
			if ($parser_ins->get_query('to') == $i) {
				$new_data .= $fields[$parser_ins->get_query('from')];
			}
			if ($parser_ins->get_query('from') != $i) {
				$new_data .= "$_\n";
			}
		} elsif (!$parser_ins->get_query('del' . $i)) {
			my $id_ins    = new webliberty::String($parser_ins->get_query('id' . $i));
			my $field_ins = new webliberty::String($parser_ins->get_query('field' . $i));

			$id_ins->create_line;
			$field_ins->create_line;

			$new_data .= $id_ins->get_string . "\t" . $field_ins->get_string . "\n";
		}

		$i++;
	}
	close(FH);

	open(FH, ">$init->{field_file}") or &error("Write Error : $init->{field_file}");
	print FH $new_data;
	close(FH);

	$lock_ins->file_unlock;

	return;
}

### ファイル追加
sub add_file {
	if ($ENV{'REQUEST_METHOD'} ne 'POST') {
		&error('不正なアクセスです。');
	}
	if (!$parser_ins->get_query('file')) {
		&error('ファイルを選択してください。');
	}

	my $file_ins = new webliberty::File($parser_ins->get_filename('file'));
	my $dir_ins  = new webliberty::String($parser_ins->get_query('dir'));

	$dir_ins->create_line;

	if (!$dir_ins->get_string) {
		&error('アップロード先を選択してください。');
	}
	if ($file_ins->get_name =~ /[^\w\d\-\_]/) {
		&error('ファイル名は半角英数字で指定してください。');
	}
	if (-e $dir_ins->get_string . $file_ins->get_name . '.' . $file_ins->get_ext) {
		&error($file_ins->get_name . '.' . $file_ins->get_ext . 'は既に存在します。ファイル名を変更してアップロードしてください。');
	}

	my $lock_ins = new webliberty::Lock($init->{lock_file});
	if (!$lock_ins->file_lock) {
		&error('ファイルがロックされています。時間をおいてもう一度登録してください。');
	}

	open(FILE, '>' . $dir_ins->get_string . $file_ins->get_name . '.' . $file_ins->get_ext) or &error('アップロードファイルが保存できません。');
	binmode(FILE);
	print FILE $parser_ins->get_filedata('file');
	close(FILE);

	$lock_ins->file_unlock;

	return;
}

### ファイル編集
sub edit_file {
	if ($ENV{'REQUEST_METHOD'} ne 'POST') {
		&error('不正なアクセスです。');
	}
	if (!$parser_ins->get_query('del')) {
		&error("削除したいファイルを選択してください。");
	}

	my $lock_ins = new webliberty::Lock($init->{lock_file});
	if (!$lock_ins->file_lock) {
		&error('ファイルがロックされています。時間をおいてもう一度登録してください。');
	}

	foreach (split(/\n/, $parser_ins->get_query('del'))) {
		unlink($_) or &error('ファイルが削除できません。');
	}

	$lock_ins->file_unlock;

	return;
}

### トップページ編集
sub update {
	if ($ENV{'REQUEST_METHOD'} ne 'POST') {
		&error('不正なアクセスです。');
	}
	if (!$init->{top_mode}) {
		&error('不正なアクセスです。');
	}

	my $break_ins = new webliberty::String($parser_ins->get_query('break'));
	my $text_ins  = new webliberty::String($parser_ins->get_query('text'));

	$break_ins->create_number;
	$text_ins->create_text;

	if (!$text_ins->get_string) {
		&error('メッセージが入力されていません。');
	}

	open(FH, ">$init->{top_file}") or &error("Write Error : $init->{top_file}");
	print FH $text_ins->get_string . "\t" . $break_ins->get_string;
	close(FH);

	return;
}

### 処理内容確認表示
sub confirm {
	if (!$parser_ins->get_query('del')) {
		&error('削除したい作品を選択してください。');
	}

	my $del_list;
	foreach (split(/\n/, $parser_ins->get_query('del'))) {
		if ($del_list) {
			$del_list .= '、';
		}

		$del_list .= "ID $_";
	}

	my $message = "$del_listの作品を削除します。よろしければ、<em>実行ボタン</em>を押してください。";

	my $skin_ins = new webliberty::Skin;
	$skin_ins->parse_skin($init->{skin_admin_header});
	$skin_ins->parse_skin($init->{skin_admin_confirm});
	$skin_ins->parse_skin($init->{skin_admin_footer});

	$skin_ins->replace_skin(
		INFO_TITLE   => $init->{title},
		INFO_FILE    => $init->{script_file},
		INFO_BACK    => $init->{back_url},
		INFO_MESSAGE => $message
	);

	my $confirm_data = "<input type=\"hidden\" name=\"exec_del\" value=\"on\" />";
	foreach (split(/\n/, $parser_ins->get_query('del'))) {
		$confirm_data .= "<input type=\"hidden\" name=\"del\" value=\"$_\" />";
	}

	print $basis_ins->header;
	print $skin_ins->get_data('header');
	print &work($skin_ins);

	print $skin_ins->get_replace_data(
		'contents',
		CONFIRM_HEADING => 'コンテンツ削除',
		CONFIRM_WORK    => $parser_ins->get_query('work'),
		CONFIRM_DATA    => $confirm_data
	);

	print $skin_ins->get_data('footer');

	return;
}

### 登録作品一覧
sub edit {
	my $message;
	if ($parser_ins->get_query('work') eq 'new') {
		$message = '作品を新規に登録しました。';
	} elsif ($parser_ins->get_query('exec_regist')) {
		$message = '作品を編集しました。';
	} elsif ($parser_ins->get_query('exec_del')) {
		$message = '作品を削除しました。';
	} else {
		$message = '作品を選択し、<em>編集ボタン</em>か<em>削除ボタン</em>を押してください。';
	}

	my($id_start, $id_end);
	if (!$init->{use_id}) {
		$id_start = '<!--';
		$id_end   = '-->';
	}

	my($field_start, $field_end);
	if (!$init->{use_field}) {
		$field_start = '<!--';
		$field_end   = '-->';
	}

	my($tag_start, $tag_end);
	if (!$init->{use_tag}) {
		$tag_start = '<!--';
		$tag_end   = '-->';
	}

	my($memo_start, $memo_end);
	if (!$init->{use_memo}) {
		$memo_start = '<!--';
		$memo_end   = '-->';
	}

	my $skin_ins = new webliberty::Skin;
	$skin_ins->parse_skin($init->{skin_admin_header});
	$skin_ins->parse_skin($init->{skin_admin_edit});
	$skin_ins->parse_skin($init->{skin_admin_footer});

	$skin_ins->replace_skin(
		INFO_TITLE   => $init->{title},
		INFO_FILE    => $init->{script_file},
		INFO_BACK    => $init->{back_url},
		INFO_MESSAGE => $message
	);

	print $basis_ins->header;
	print $skin_ins->get_data('header');
	print &work($skin_ins);

	print $skin_ins->get_replace_data(
		'edit_head',
		EDIT_ID_START    => $id_start,
		EDIT_ID_END      => $id_end,
		EDIT_FIELD_START => $field_start,
		EDIT_FIELD_END   => $field_end,
		EDIT_TAG_START   => $tag_start,
		EDIT_TAG_END     => $tag_end,
		EDIT_MEMO_START  => $memo_start,
		EDIT_MEMO_END    => $memo_end
	);

	open(FH, $init->{log_file}) or &error("Read Error : $init->{log_file}");
	while (<FH>) {
		chomp;
		my($id, $stat, $break, $field, $tag, $date, $subj, $memo, $text) = split(/\t/);

		print $skin_ins->get_replace_data(
			'edit',
			&article($id, $stat, $break, $field, $tag, $date, $subj, $memo, $text)
		);
	}
	close(FH);

	print $skin_ins->get_replace_data(
		'edit_foot',
		EDIT_ID_START    => $id_start,
		EDIT_ID_END      => $id_end,
		EDIT_FIELD_START => $field_start,
		EDIT_FIELD_END   => $field_end,
		EDIT_TAG_START   => $tag_start,
		EDIT_TAG_END     => $tag_end,
		EDIT_MEMO_START  => $memo_start,
		EDIT_MEMO_END    => $memo_end
	);

	print $skin_ins->get_data('footer');

	return;
}

### 作品登録フォーム表示
sub form {
	my($form_work, $form_id, $form_stat, $form_break, $form_field, $form_tag, $form_date, $form_subj, $form_memo, $form_text, $form_new_start, $form_new_end, $form_edit_start, $form_edit_end);

	if ($parser_ins->get_query('work') eq 'edit') {
		if (!$parser_ins->get_query('edit')) {
			&error('編集したい作品を選択してください。');
		}

		$form_new_start = '<!--';
		$form_new_end   = '-->';

		$form_work = 'edit';

		my $flag;

		open(FH, $init->{log_file}) or &error("Read Error : $init->{log_file}");
		while (<FH>) {
			chomp;
			my($id, $stat, $break, $field, $tag, $date, $subj, $memo, $text) = split(/\t/);

			if ($parser_ins->get_query('edit') eq $id) {
				my $memo_ins = new webliberty::String($memo);
				my $text_ins = new webliberty::String($text);

				$memo_ins->create_text;
				$text_ins->create_text;

				$form_id    = $id;
				$form_stat  = $stat;
				$form_break = $break;
				$form_field = $field;
				$form_tag   = $tag;
				$form_date  = $date;
				$form_subj  = $subj;
				$form_memo  = $memo_ins->create_plain;
				$form_text  = $text_ins->create_plain;

				$form_tag =~ s/<>/,/g;

				$flag = 1;

				last;
			}
		}
		close(FH);

		if (!$flag) {
			&error('指定された作品は存在しません。');
		}
	} else {
		$form_edit_start = '<!--';
		$form_edit_end   = '-->';

		$form_work = 'new';

		$form_stat  = $init->{default_stat};
		$form_break = $init->{default_break};

		my($sec, $min, $hour, $day, $mon, $year) = localtime(time);
		$form_date = sprintf("%04d%02d%02d%02d%02d", $year + 1900, $mon + 1, $day, $hour, $min);
	}

	my($form_id_start, $form_id_end);
	if (!$init->{use_id}) {
		$form_id_start = '<!--';
		$form_id_end   = '-->';
	}

	if ($form_stat) {
		$form_stat = ' checked="checked"';
	} else {
		$form_stat = '';
	}
	if ($form_break) {
		$form_break = ' checked="checked"';
	} else {
		$form_break = '';
	}

	my($form_field_start, $form_field_end);
	if ($init->{use_field}) {
		my $field_list;

		open(FH, $init->{field_file}) or &error("Read Error : $init->{field_file}");
		while (<FH>) {
			chomp;
			my($id, $field) = split(/\t/);
	
			if ($form_field eq $id) {
				$field_list .= "<option value=\"$id\" selected=\"selected\">$field</option>";
			} else {
				$field_list .= "<option value=\"$id\">$field</option>";
			}
		}
		close(FH);

		if (!$field_list) {
			$field_list = '<option value="">分類が登録されていません</option>';
		} else {
			$field_list = "<option value=\"\">未分類</option>$field_list";
		}
		$form_field = "<select name=\"field\" xml:lang=\"ja\" lang=\"ja\">$field_list</select>";
	} else {
		$form_field_start = '<!--';
		$form_field_end   = '-->';
	}

	my($form_tag_start, $form_tag_end);
	if (!$init->{use_tag}) {
		$form_tag_start = '<!--';
		$form_tag_end   = '-->';
	}

	if ($form_date =~ /^(\d\d\d\d)(\d\d)(\d\d)(\d\d)(\d\d)$/) {
		my $form_year   = $1;
		my $form_month  = $2;
		my $form_day    = $3;
		my $form_hour   = $4;
		my $form_minute = $5;

		$form_date = '<select name="year" xml:lang="ja" lang="ja">';
		foreach ($form_year - 5 .. $form_year + 5) {
			if ($form_year == $_) {
				$form_date .= "<option value=\"$_\" selected=\"selected\">$_年</option>";
			} else {
				$form_date .= "<option value=\"$_\">$_年</option>";
			}
		}
		$form_date .= '</select>';

		$form_date .= '<select name="month" xml:lang="ja" lang="ja">';
		foreach (1 .. 12) {
			my $month = sprintf("%02d", $_);
			if ($month == $form_month) {
				$form_date .= "<option value=\"$month\" selected=\"selected\">$_月</option>";
			} else {
				$form_date .= "<option value=\"$month\">$_月</option>";
			}
		}
		$form_date .= '</select>';

		$form_date .= '<select name="day" xml:lang="ja" lang="ja">';
		foreach (1 .. 31) {
			my $day = sprintf("%02d", $_);
			if ($day == $form_day) {
				$form_date .= "<option value=\"$day\" selected=\"selected\">$_日</option>";
			} else {
				$form_date .= "<option value=\"$day\">$_日</option>";
			}
		}
		$form_date .= '</select>';

		$form_date .= '<select name="hour" xml:lang="ja" lang="ja">';
		foreach (0 .. 23) {
			my $hour = sprintf("%02d", $_);
			if ($hour == $form_hour) {
				$form_date .= "<option value=\"$hour\" selected=\"selected\">$_時</option>";
			} else {
				$form_date .= "<option value=\"$hour\">$_時</option>";
			}
		}
		$form_date .= '</select>';

		$form_date .= '<select name="minute" xml:lang="ja" lang="ja">';
		foreach (0 .. 59) {
			my $minute = sprintf("%02d", $_);
			if ($minute == $form_minute) {
				$form_date .= "<option value=\"$minute\" selected=\"selected\">$_分</option>";
			} else {
				$form_date .= "<option value=\"$minute\">$_分</option>";
			}
		}
		$form_date .= '</select>';
	}

	my($form_memo_start, $form_memo_end);
	if (!$init->{use_memo}) {
		$form_memo_start = '<!--';
		$form_memo_end   = '-->';
	}

	my $form_file;
	foreach my $dir (@{$init->{file_dir}}) {
		if (!$dir) {
			next;
		}

		opendir(DIR, $dir) or &error("Read Error : $dir");
		my @files = sort { $a cmp $b } grep { m/\w+\.\w+/g } readdir(DIR);
		close(DIR);

		$form_file .= "<optgroup label=\"$dir\">";

		foreach (@files) {
			my $file_ins = new webliberty::File("$dir$_");

			my $tag;
			if ($file_ins->get_ext eq 'gif' or $file_ins->get_ext eq 'png' or $file_ins->get_ext eq 'jpeg' or $file_ins->get_ext eq 'jpg' or $file_ins->get_ext eq 'jpe') {
				my($width, $height) = $file_ins->get_size;

				$tag = "&lt;img src=&quot;$dir$_&quot; alt=&quot;$_&quot; width=&quot;$width&quot; height=&quot;$height&quot; /&gt;";
			} else {
				$tag = "&lt;a href=&quot;$dir$_&quot;&gt;$_&lt;/a&gt;";
			}

			$form_file .= "<option value=\"$tag\">$_</option>";
		}

		$form_file .= "</optgroup>";
	}
	$form_file = "<select name=\"file\" xml:lang=\"ja\" lang=\"ja\"><option value=\"\">選択してください</option>$form_file</select>";

	my $skin_ins = new webliberty::Skin;
	$skin_ins->parse_skin($init->{skin_admin_header});
	$skin_ins->parse_skin($init->{skin_admin_form});
	$skin_ins->parse_skin($init->{skin_admin_footer});

	$skin_ins->replace_skin(
		INFO_TITLE  => $init->{title},
		INFO_FILE   => $init->{script_file},
		INFO_BACK   => $init->{back_url}
	);

	print $basis_ins->header;
	print $skin_ins->get_data('header');
	print &work($skin_ins);

	print $skin_ins->get_replace_data(
		'form',
		FORM_WORK        => $parser_ins->get_query('work'),
		FORM_EDIT        => $parser_ins->get_query('edit'),
		FORM_ID          => $form_id,
		FORM_ID_START    => $form_id_start,
		FORM_ID_END      => $form_id_end,
		FORM_STAT        => $form_stat,
		FORM_BREAK       => $form_break,
		FORM_FIELD       => $form_field,
		FORM_FIELD_START => $form_field_start,
		FORM_FIELD_END   => $form_field_end,
		FORM_TAG         => $form_tag,
		FORM_TAG_START   => $form_tag_start,
		FORM_TAG_END     => $form_tag_end,
		FORM_DATE        => $form_date,
		FORM_SUBJ        => $form_subj,
		FORM_MEMO        => $form_memo,
		FORM_MEMO_START  => $form_memo_start,
		FORM_MEMO_END    => $form_memo_end,
		FORM_TEXT        => $form_text,
		FORM_FILE        => $form_file,
		FORM_NEW_START   => $form_new_start,
		FORM_NEW_END     => $form_new_end,
		FORM_EDIT_START  => $form_edit_start,
		FORM_EDIT_END    => $form_edit_end
	);

	print $skin_ins->get_data('footer');

	return;
}

### 分類管理画面
sub field {
	my $message;
	if ($parser_ins->get_query('exec_add')) {
		$message = '分類を新規に登録しました。';
	} elsif ($parser_ins->get_query('exec_edit')) {
		$message = '分類を編集しました。';
	} else {
		$message = 'コンテンツ登録時の分類を設定する事ができます。';
	}

	my $skin_ins = new webliberty::Skin;
	$skin_ins->parse_skin($init->{skin_admin_header});
	$skin_ins->parse_skin($init->{skin_admin_field});
	$skin_ins->parse_skin($init->{skin_admin_footer});

	$skin_ins->replace_skin(
		INFO_TITLE   => $init->{title},
		INFO_FILE    => $init->{script_file},
		INFO_BACK    => $init->{back_url},
		INFO_MESSAGE => $message
	);

	print $basis_ins->header;
	print $skin_ins->get_data('header');
	print &work($skin_ins);

	print $skin_ins->get_data('contents');
	print $skin_ins->get_data('field_head');

	my $i = 0;

	open(FH, $init->{field_file}) or &error("Read Error : $init->{field_file}");
	while (<FH>) {
		chomp;
		my($id, $field) = split(/\t/);

		print $skin_ins->get_replace_data(
			'field',
			FIELD_ID   => $id,
			FIELD_NAME => $field,
			FIELD_NO   => $i
		);

		$i++;
	}
	close(FH);

	print $skin_ins->get_data('field_foot');
	print $skin_ins->get_data('footer');

	return;
}

### ファイル管理画面
sub file {
	my $message;
	if ($parser_ins->get_query('exec_add')) {
		$message = 'ファイルをアップロードしました。';
	} elsif ($parser_ins->get_query('exec_edit')) {
		$message = 'ファイルを削除しました。';
	} else {
		$message = 'ファイルを管理する事ができます。';
	}

	my $skin_ins = new webliberty::Skin;
	$skin_ins->parse_skin($init->{skin_admin_header});
	$skin_ins->parse_skin($init->{skin_admin_file});
	$skin_ins->parse_skin($init->{skin_admin_footer});

	$skin_ins->replace_skin(
		INFO_TITLE   => $init->{title},
		INFO_FILE    => $init->{script_file},
		INFO_BACK    => $init->{back_url},
		INFO_MESSAGE => $message
	);

	print $basis_ins->header;
	print $skin_ins->get_data('header');
	print &work($skin_ins);

	my $form_dir;
	foreach (@{$init->{file_dir}}) {
		if (!$_) {
			next;
		}
		$form_dir .= "<option value=\"$_\">$_</option>";
	}
	if (!$form_dir) {
		$form_dir = '<option value="">アップロード先が登録されていません</option>';
	}
	$form_dir = "<select name=\"dir\" xml:lang=\"ja\" lang=\"ja\">$form_dir</select>";

	print $skin_ins->get_replace_data(
		'contents',
		FILE_DIR => $form_dir
	);
	print $skin_ins->get_data('file_head');

	foreach my $dir (@{$init->{file_dir}}) {
		if (!$dir) {
			next;
		}

		opendir(DIR, $dir) or &error("Read Error : $dir");
		my @files = sort { $a cmp $b } grep { m/\w+\.\w+/g } readdir(DIR);
		close(DIR);

		foreach (@files) {
			print $skin_ins->get_replace_data(
				'file',
				FILE_DIR  => $dir,
				FILE_NAME => $_
			);
		}
	}

	print $skin_ins->get_data('file_foot');
	print $skin_ins->get_data('footer');

	return;
}

### トップページ管理
sub top {
	if (!$init->{top_mode}) {
		&error('不正なアクセスです。');
	}

	my $message;
	if ($parser_ins->get_query('exec_regist')) {
		$message = 'メッセージを設定しました。';
	} else {
		$message = 'トップページに表示するメッセージを設定します。';
	}

	open(FH, "$init->{top_file}") or &error("Read Error : $init->{top_file}");
	my $log = <FH>;
	close(FH);

	my($form_text, $form_break) = split(/\t/, $log);

	if ($form_break) {
		$form_break = ' checked="checked"';
	} else {
		$form_break = '';
	}

	my $text_ins = new webliberty::String($form_text);
	$text_ins->create_text;

	my $form_file;
	foreach my $dir (@{$init->{file_dir}}) {
		if (!$dir) {
			next;
		}

		opendir(DIR, $dir) or &error("Read Error : $dir");
		my @files = sort { $a cmp $b } grep { m/\w+\.\w+/g } readdir(DIR);
		close(DIR);

		$form_file .= "<optgroup label=\"$dir\">";

		foreach (@files) {
			my $file_ins = new webliberty::File("$dir$_");

			my $tag;
			if ($file_ins->get_ext eq 'gif' or $file_ins->get_ext eq 'png' or $file_ins->get_ext eq 'jpeg' or $file_ins->get_ext eq 'jpg' or $file_ins->get_ext eq 'jpe') {
				my($width, $height) = $file_ins->get_size;

				$tag = "&lt;img src=&quot;$dir$_&quot; alt=&quot;$_&quot; width=&quot;$width&quot; height=&quot;$height&quot; /&gt;";
			} else {
				$tag = "&lt;a href=&quot;$dir$_&quot;&gt;$_&lt;/a&gt;";
			}

			$form_file .= "<option value=\"$tag\">$_</option>";
		}

		$form_file .= "</optgroup>";
	}
	$form_file = "<select name=\"file\" xml:lang=\"ja\" lang=\"ja\"><option value=\"\">選択してください</option>$form_file</select>";

	my $skin_ins = new webliberty::Skin;
	$skin_ins->parse_skin($init->{skin_admin_header});
	$skin_ins->parse_skin($init->{skin_admin_top});
	$skin_ins->parse_skin($init->{skin_admin_footer});

	$skin_ins->replace_skin(
		INFO_TITLE   => $init->{title},
		INFO_FILE    => $init->{script_file},
		INFO_BACK    => $init->{back_url},
		INFO_MESSAGE => $message
	);

	print $basis_ins->header;
	print $skin_ins->get_data('header');
	print &work($skin_ins);

	print $skin_ins->get_replace_data(
		'form',
		FORM_TEXT  => $text_ins->create_plain,
		FORM_FILE  => $form_file,
		FORM_BREAK => $form_break
	);

	print $skin_ins->get_data('footer');

	return;
}

### ログイン画面
sub login {
	my $message;
	if ($parser_ins->get_query('admin_pwd')) {
		$message = 'パスワードが違います。';
	} else {
		$message = '管理者パスワードを入力してください。';
	}

	my $skin_ins = new webliberty::Skin;
	$skin_ins->parse_skin($init->{skin_admin});

	$skin_ins->replace_skin(
		INFO_TITLE   => $init->{title},
		INFO_FILE    => $init->{script_file},
		INFO_BACK     => $init->{back_url},
		INFO_MESSAGE => $message
	);

	print $basis_ins->header;
	print $skin_ins->get_data('login_head');

	my $work_data;
	$work_data  = "new\t作品登録\n";
	$work_data .= "edit\t作品管理\n";
	if ($init->{use_field}) {
		$work_data .= "field\t分類管理\n";
	}
	$work_data .= "file\tファイル管理\n";
	if ($init->{top_mode}) {
		$work_data .= "top\tトップページ管理\n";
	}

	foreach (split(/\n/, $work_data)) {
		my($work_id, $work_name) = split(/\t/);

		print $skin_ins->get_replace_data(
			'login',
			WORK_ID   => $work_id,
			WORK_NAME => $work_name
		);
	}

	print $skin_ins->get_data('login_foot');

	return;
}

### 作品検索
sub search {
	my $word;
	if ($agent eq 'mobile') {
		$word = Jcode->new($parser_ins->get_query('word'), 'sjis')->utf8;
	} else {
		$word = $parser_ins->get_query('word');
	}

	my($cond_and, $cond_or);
	if ($parser_ins->get_query('cond') eq 'or') {
		$cond_and = '';
		$cond_or  = ' selected="selected"';
	} else {
		$cond_and = ' selected="selected"';
		$cond_or  = '';
	}

	my $skin_ins = new webliberty::Skin;
	if ($agent eq 'mobile') {
		$skin_ins->parse_skin($init->{skin_mobile_search});
	} else {
		$skin_ins->parse_skin($init->{skin_search});
	}

	$skin_ins->replace_skin(
		INFO_TITLE    => $init->{title},
		INFO_FILE     => $init->{script_file},
		INFO_BACK     => $init->{back_url},
		INFO_WORD     => $word,
		INFO_COND_AND => $cond_and,
		INFO_COND_OR  => $cond_or
	);

	my $html;
	$html .= $skin_ins->get_data('header');

	$html .= $skin_ins->get_data('contents');

	$html .= $skin_ins->get_replace_data(
		'gallery_head',
		GALLERY_HEAD => '検索結果'
	);

	my $i;

	if ($word) {
		my $words = $word;
		$words =~ s/　/ /g;
		my @words = split(/\s+/, $words);

		open(FH, $init->{log_file}) or &error("Read Error : $init->{log_file}");
		while (<FH>) {
			chomp;
			my($id, $stat, $break, $field, $tag, $date, $subj, $memo, $text) = split(/\t/);

			if (!$stat) {
				next;
			}

			my $flag;

			foreach my $word (@words) {
				$word = lc($word);
				my $string = lc("$id\t$field\t$tag\t$date\t$subj\t$memo\t$text");

				if (index($string, $word) >= 0) {
					$flag = 1;
					if ($parser_ins->get_query('cond') eq 'or') {
						last;
					}
				} else {
					if ($parser_ins->get_query('cond') eq 'and') {
						$flag = 0;

						last;
					}
				}
			}
			if (!$flag) {
				next;
			}

			$html .= $skin_ins->get_replace_data(
				'gallery',
				&article($id, $stat, $break, $field, $tag, $date, $subj, $memo, $text)
			);

			$i++;
		}
		close(FH);
	}

	if (!$i) {
		my $message;
		if ($word) {
			$message = '条件に一致する作品はありませんでした。';
		} else {
			$message = '検索条件を指定してください。';
		}

		$html .= $skin_ins->get_replace_data(
			'gallery_void',
			INFO_MESSAGE => $message
		);
	}

	$html .= $skin_ins->get_data('gallery_foot');

	$html .= &navigation($skin_ins);
	$html .= $skin_ins->get_data('footer');

	if ($agent eq 'mobile') {
		my($html, $length) = &get_mobile_data($html);

		print "Content-Type: text/html; charset=Shift_JIS\n";
		if ($ENV{'HTTP_USER_AGENT'} =~ /J-PHONE|KDDI-|UP\.Browser/i) {
			print "Pragma: no-cache\n";
			print "Cache-Control: no-cache\n\n";
		} else {
			print "Content-Length: $length\n\n";
		}
		print $html;
	} else {
		print $basis_ins->header;
		print $html;
	}

	return;
}

### 作品表示
sub view {
	my $skin_ins = new webliberty::Skin;
	if ($agent eq 'mobile') {
		$skin_ins->parse_skin($init->{skin_mobile_view});
	} else {
		$skin_ins->parse_skin($init->{skin_view});
	}

	$skin_ins->replace_skin(
		INFO_TITLE => $init->{title},
		INFO_FILE  => $init->{script_file},
		INFO_BACK  => $init->{back_url}
	);

	my $html;
	$html .= $skin_ins->get_data('header');

	my($prev_id, $next_id, $this_field, $prev_field, $next_field, $flag);

	open(FH, $init->{log_file}) or &error("Read Error : $init->{log_file}");
	while (<FH>) {
		chomp;
		my($id, $stat, $break, $field, $tag, $date, $subj, $memo, $text) = split(/\t/);

		if (!$stat) {
			next;
		}

		if ($flag) {
			$next_id    = $id;
			$next_field = $field;

			last;
		}

		if ($parser_ins->get_query('id') eq $id) {
			$html .= $skin_ins->get_replace_data(
				'gallery',
				&article($id, $stat, $break, $field, $tag, $date, $subj, $memo, $text)
			);

			$this_field = $field;
			$flag       = 1;
		}

		if (!$flag) {
			$prev_id    = $id;
			$prev_field = $field;
		}
	}
	close(FH);

	if ($flag) {
		my($prev_start, $prev_end);
		if ($prev_id and $this_field eq $prev_field) {
			$prev_start = "<a href=\"$init->{script_file}?mode=view&amp;id=$prev_id\">";
			$prev_end   = '</a>';
		}

		my($next_start, $next_end);
		if ($next_id and $this_field eq $next_field) {
			$next_start = "<a href=\"$init->{script_file}?mode=view&amp;id=$next_id\">";
			$next_end   = '</a>';
		}

		$html .= $skin_ins->get_replace_data(
			'navi',
			NAVI_PREV_START => $prev_start,
			NAVI_PREV_END   => $prev_end,
			NAVI_NEXT_START => $next_start,
			NAVI_NEXT_END   => $next_end
		);
	} else {
		$html .= $skin_ins->get_replace_data(
			'gallery',
			&article('', 1, 1, '', '', '', 'Nothing', '該当する作品はありません。')
		);
		$html .= $skin_ins->get_data(
			'navi'
		);
	}

	$html .= &navigation($skin_ins);
	$html .= $skin_ins->get_data('footer');

	if ($agent eq 'mobile') {
		my($html, $length) = &get_mobile_data($html);

		print "Content-Type: text/html; charset=Shift_JIS\n";
		if ($ENV{'HTTP_USER_AGENT'} =~ /J-PHONE|KDDI-|UP\.Browser/i) {
			print "Pragma: no-cache\n";
			print "Cache-Control: no-cache\n\n";
		} else {
			print "Content-Length: $length\n\n";
		}
		print $html;
	} else {
		print $basis_ins->header;
		print $html;
	}

	return;
}

### 題名一覧表示
sub list {
	my $skin_ins = new webliberty::Skin;
	if ($agent eq 'mobile') {
		$skin_ins->parse_skin($init->{skin_mobile_list});
	} else {
		$skin_ins->parse_skin($init->{skin_list});
	}

	$skin_ins->replace_skin(
		INFO_TITLE => $init->{title},
		INFO_FILE  => $init->{script_file},
		INFO_BACK  => $init->{back_url}
	);

	my(%fields, $head);
	if ($parser_ins->get_query('field')) {
		open(FH, $init->{field_file}) or &error("Read Error : $init->{field_file}");
		while (<FH>) {
			chomp;
			my($id, $field) = split(/\t/);

			$fields{$id} = $field;
		}
		close(FH);

		$head = $fields{$parser_ins->get_query('field')};
	} elsif ($parser_ins->get_query('tag')) {
		my $tag_ins = new webliberty::String($parser_ins->get_query('tag'));
		$head = $tag_ins->create_line;
	} else {
		$head = '未分類';
	}
	if (!$head) {
		$head = 'Nothing';
	}

	my $html;
	$html .= $skin_ins->get_data('header');

	$html .= $skin_ins->get_replace_data(
		'gallery_head',
		GALLERY_HEAD => $head
	);

	my $flag;

	open(FH, $init->{log_file}) or &error("Read Error : $init->{log_file}");
	while (<FH>) {
		chomp;
		my($id, $stat, $break, $field, $tag, $date, $subj, $memo, $text) = split(/\t/);

		if (!$stat) {
			next;
		}

		my $word = $parser_ins->get_query('tag');

		if ($parser_ins->get_query('field') and $parser_ins->get_query('field') ne $field) {
			next;
		}
		if ($parser_ins->get_query('tag') and $tag !~ /(^|<>)$word(<>|$)/) {
			next;
		}

		$html .= $skin_ins->get_replace_data(
			'gallery',
			&article($id, $stat, $break, $field, $tag, $date, $subj, $memo, $text)
		);

		$flag = 1;
	}
	close(FH);

	if (!$flag) {
		$html .= $skin_ins->get_replace_data(
			'gallery',
			&article('', 1, 1, '', '', '', 'Nothing', '該当する作品はありません。')
		);
	}

	$html .= $skin_ins->get_data('gallery_foot');

	$html .= &navigation($skin_ins);
	$html .= $skin_ins->get_data('footer');

	if ($agent eq 'mobile') {
		my($html, $length) = &get_mobile_data($html);

		print "Content-Type: text/html; charset=Shift_JIS\n";
		if ($ENV{'HTTP_USER_AGENT'} =~ /J-PHONE|KDDI-|UP\.Browser/i) {
			print "Pragma: no-cache\n";
			print "Cache-Control: no-cache\n\n";
		} else {
			print "Content-Length: $length\n\n";
		}
		print $html;
	} else {
		print $basis_ins->header;
		print $html;
	}

	return;
}

### 初期画面表示
sub gallery {
	my $top_message;
	if ($init->{top_mode}) {
		open(FH, "$init->{top_file}") or &error("Read Error : $init->{top_file}");
		my $log = <FH>;
		close(FH);

		my($form_text, $form_break) = split(/\t/, $log);

		my $text_ins = new webliberty::String($form_text);
		if ($form_break) {
			$text_ins->set_string('<p>' . $text_ins->get_string . '</p>');
		} else {
			$text_ins->replace_string('<br />', "\n");
		}
		$top_message = $text_ins->permit_html;
	}

	my $skin_ins = new webliberty::Skin;
	if ($agent eq 'mobile') {
		$skin_ins->parse_skin($init->{skin_mobile_gallery});
	} else {
		$skin_ins->parse_skin($init->{skin_gallery});
	}

	$skin_ins->replace_skin(
		INFO_TITLE => $init->{title},
		INFO_FILE  => $init->{script_file},
		INFO_BACK  => $init->{back_url}
	);

	my $html;
	$html .= $skin_ins->get_data('header');

	$html .= $skin_ins->get_replace_data(
		'gallery',
		TOP_MESSAGE => $top_message
	);

	$html .= &navigation($skin_ins);
	$html .= $skin_ins->get_data('footer');

	if ($agent eq 'mobile') {
		my($html, $length) = &get_mobile_data($html);

		print "Content-Type: text/html; charset=Shift_JIS\n";
		if ($ENV{'HTTP_USER_AGENT'} =~ /J-PHONE|KDDI-|UP\.Browser/i) {
			print "Pragma: no-cache\n";
			print "Cache-Control: no-cache\n\n";
		} else {
			print "Content-Length: $length\n\n";
		}
		print $html;
	} else {
		print $basis_ins->header;
		print $html;
	}

	return;
}

### 作品表示
sub article {
	my($id, $stat, $break, $field, $tag, $date, $subj, $memo, $text) = @_;

	my($id_start, $id_end);
	if (!$init->{use_id}) {
		$id_start = '<!--';
		$id_end   = '-->';
	}

	if ($stat) {
		$stat = '公開';
	} else {
		$stat = '下書き';
	}

	my(%fields, $field_start, $field_end);
	if ($init->{use_field}) {
		open(FIELD, $init->{field_file}) or &error("Read Error : $init->{field_file}");
		while (<FIELD>) {
			chomp;
			my($id, $field) = split(/\t/);
	
			$fields{$id} = $field;
		}
		close(FIELD);
	} else {
		$field_start = '<!--';
		$field_end   = '-->';
	}

	my($tags, $tag_start, $tag_end);
	if ($init->{use_tag} and $tag) {
		foreach my $word (split(/<>/, $tag)) {
			my $encoder_ins = new webliberty::Encoder($word);
			$encoder_ins->url_encode;

			$tags .= "[<a href=\"$init->{script_file}?mode=list&amp;tag=" . $encoder_ins->get_string . "\">$word</a>]";
		}
	} else {
		$tag_start = '<!--';
		$tag_end   = '-->';
	}

	if (!$date) {
		my($sec, $min, $hour, $day, $mon, $year) = localtime(time);
		$date = sprintf("%04d%02d%02d%02d%02d", $year + 1900, $mon + 1, $day, $hour, $min);
	}

	my($year, $month, $day, $hour, $minute, $week, $new_start, $new_end);
	if ($date =~ /^(\d\d\d\d)(\d\d)(\d\d)(\d\d)(\d\d)$/) {
		my $day_ins = new webliberty::Date;

		$year   = $1;
		$month  = $2;
		$day    = $3;
		$hour   = $4;
		$minute = $5;
		$week   = ${$init->{weeks}}[$day_ins->get_week("$1-$2-$3")];
	}
	if ($date =~ /^(\d\d\d\d)(\d\d)(\d\d)(\d\d)(\d\d)$/) {
		my($sec, $min, $hour, $day, $mon, $year, $week) = localtime(time);
		my $day_ins = new webliberty::Date;

		if ($day_ins->get_interval(sprintf("%04d-%02d-%02d", $year + 1900, $mon + 1, $day), "$1-$2-$3") >= $init->{new_days}) {
			$new_start = '<!--';
			$new_end   = '-->';
		}
	}

	my($memo_start, $memo_end);
	if ($init->{use_memo} and $memo) {
		my $memo_ins = new webliberty::String($memo);
		$memo = $memo_ins->permit_html;
	} else {
		$memo_start = '<!--';
		$memo_end   = '-->';
	}

	my $text_ins = new webliberty::String($text);
	if ($break) {
		$text_ins->set_string('<p>' . $text_ins->get_string . '</p>');
	} else {
		$text_ins->replace_string('<br />', "\n");
	}
	$text_ins->permit_html;

	my $cookie_ins = new webliberty::Cookie($init->{cookie_admin}, $init->{des_key});

	my($admin_start, $admin_end);
	if (!$cookie_ins->get_cookie('admin_pwd')) {
		$admin_start = '<!--';
		$admin_end   = '-->';
	}

	return(
		GALLERY_ID          => $id,
		GALLERY_ID_START    => $id_start,
		GALLERY_ID_END      => $id_end,
		GALLERY_STAT        => $stat,
		GALLERY_BREAK       => $break,
		GALLERY_FID         => $field,
		GALLERY_FIELD       => $fields{$field},
		GALLERY_FIELD_START => $field_start,
		GALLERY_FIELD_END   => $field_end,
		GALLERY_TAG         => $tags,
		GALLERY_TAG_START   => $tag_start,
		GALLERY_TAG_END     => $tag_end,
		GALLERY_YEAR        => $year,
		GALLERY_MONTH       => $month,
		GALLERY_DAY         => $day,
		GALLERY_HOUR        => $hour,
		GALLERY_MINUTE      => $minute,
		GALLERY_WEEK        => $week,
		GALLERY_SUBJ        => $subj,
		GALLERY_MEMO        => $memo,
		GALLERY_MEMO_START  => $memo_start,
		GALLERY_MEMO_END    => $memo_end,
		GALLERY_TEXT        => $text_ins->get_string,
		GALLERY_NEW_START   => $new_start,
		GALLERY_NEW_END     => $new_end,
		GALLERY_ADMIN_START => $admin_start,
		GALLERY_ADMIN_END   => $admin_end
	);
}

### ナビゲーション表示
sub navigation {
	my($skin_ins) = @_;

	my $html;

	if ($init->{use_field}) {
		my %fields;
		open(FH, $init->{field_file}) or &error("Read Error : $init->{field_file}");
		while (<FH>) {
			chomp;
			my($id, $field) = split(/\t/);

			$fields{$id} = $field;
		}
		close(FH);

		$html .= $skin_ins->get_data('list_head');

		my($flag, $prev_field);

		open(FH, $init->{log_file}) or &error("Read Error : $init->{log_file}");
		while (<FH>) {
			chomp;
			my($id, $stat, $break, $field, $tag, $date, $subj, $memo, $text) = split(/\t/);

			if (!$stat) {
				next;
			}

			if (!$flag or ($prev_field and $prev_field ne $field)) {
				if (!$field) {
					$field = '_';
				}

				if ($flag) {
					$html .= $skin_ins->get_data('field_foot');
				}

				if ($field eq '_') {
					$html .= $skin_ins->get_replace_data(
						'field_head',
						FIELD_ID   => '',
						FIELD_NAME => '未分類'
					);
				} else {
					$html .= $skin_ins->get_replace_data(
						'field_head',
						FIELD_ID   => $field,
						FIELD_NAME => $fields{$field}
					);
				}

				$flag = 1;
			}

			$html .= $skin_ins->get_replace_data(
				'list',
				&article($id, $stat, $break, $field, $tag, $date, $subj, $memo, $text)
			);

			$prev_field = $field;
		}
		close(FH);

		if ($flag) {
			$html .= $skin_ins->get_data('field_foot');
		}

		$html .= $skin_ins->get_data('list_foot');
	}

	if ($init->{use_tag}) {
		$html .= $skin_ins->get_data('tag_head');

		open(FH, $init->{tag_file}) or &error("Read Error : $init->{tag_file}");
		while (<FH>) {
			chomp;
			my($tag, $size) = split(/\t/);

			my $encoder_ins = new webliberty::Encoder($tag);
			$encoder_ins->url_encode;

			$html .= $skin_ins->get_replace_data(
				'tag',
				TAG_NAME => $tag,
				TAG_LINK => $init->{script_file} . '?mode=list&amp;tag=' . $encoder_ins->get_string,
				TAG_SIZE => $size
			);
		}
		close(FH);

		$html .= $skin_ins->get_data('tag_foot');
	}

	return $html;
}

### アイテム並び替え
sub sort_item {
	my @item = @_;

	my %fields;
	my $i = 0;
	open(FH, $init->{field_file}) or &error("Read Error : $init->{field_file}");
	while (<FH>) {
		chomp;
		my($id, $field) = split(/\t/);

		$fields{$id} = $i++;
	}
	close(FH);

	my @fields = map { (split(/\t/))[3] } @item;
	@item = @item[sort { $fields{$fields[$a]} <=> $fields{$fields[$b]} } (0 .. $#fields)];

	return(@item);
}

### 解析フォーム出力
sub work {
	my($skin_ins) = @_;

	my $skin_data = $skin_ins->get_data('work_head');

	my $work_data;
	$work_data .= "new\t作品登録\n";
	$work_data .= "edit\t作品管理\n";
	if ($init->{use_field}) {
		$work_data .= "field\t分類管理\n";
	}
	$work_data .= "file\tファイル管理\n";
	if ($init->{top_mode}) {
		$work_data .= "top\tトップページ管理\n";
	}
	$work_data .= "logout\tログアウト\n";

	foreach (split(/\n/, $work_data)) {
		my($work_id, $work_name) = split(/\t/);

		if ($parser_ins->get_query('work') eq $work_id) {
			$skin_data .= $skin_ins->get_replace_data(
				'work_selected',
				WORK_ID   => $work_id,
				WORK_NAME => $work_name
			);
		} else {
			$skin_data .= $skin_ins->get_replace_data(
				'work',
				WORK_ID   => $work_id,
				WORK_NAME => $work_name
			);
		}
	}

	$skin_data .= $skin_ins->get_data('work_foot');

	return $skin_data;
}

### エラー表示
sub error {
	my($message) = @_;

	my $skin_ins = new webliberty::Skin;
	$skin_ins->parse_skin($init->{skin_error});

	$skin_ins->replace_skin(
		INFO_ERROR => $message
	);

	print $basis_ins->header;
	print $skin_ins->get_data('_all');

	exit;
}

### 携帯用データ作成
sub get_mobile_data {
	my($data) = @_;

	$data =~ s/\xEF\xBD\x9E/\xE3\x80\x9C/g;
	$data = Jcode->new($data, 'utf8')->sjis;

	return($data, length($data) + ($data =~ s/\n/\n/g));
}
